#include "CharData.h"

//Initializing
CharData::CharData()
{
   charWidth = 0;
   charHeight = 0;
   wordWidth = 0;
   image = NULL;
   words = 0;
   style = "0x";
   lines = 0;
   for(int i = 0; i < MAX_WORDS_PER_CHAR; i++) wordArray[i] = 0;
}

CharData::~CharData()
{
   if(image) delete(image);
}

CharData*
CharData::withCharExtent(int w, int h)
{
   charWidth = w;
   charHeight = h;
   if(image) delete image;
   image = new char[charWidth * charHeight];
   return this;
}

CharData*
CharData::withWordWidth(int w)
{
   wordWidth = w;
   return this;
}

CharData*
CharData::withMapping(unsigned char* to, unsigned long* as, int sizeInWords, int sizeInBits)
{
   mapTo = to;
   mapAs = as;
   mapSizeInWords = sizeInWords;
   mapSizeInBits = sizeInBits;
   return this;
}

CharData*
CharData::addPixelLine(png_byte* line)
{
   int i = charWidth;
   int x = lines * charWidth;

   while(i-- > 0)
   {
      image[x + i] = (line[i] > 0.5? 1: 0);
   }
   lines++;
   return this;
}

//Printing
CharData*
CharData::setStyle(const char* x)
{
   style = x;
   return this;
}

CharData*
CharData::print(FILE* f)
{
   int i;
   int w = (wordWidth + 3) / 4; //1234->1, 5678->2, 9ABC->3,.. 
   for(i = 0; i < words; i++)
   {
      fprintf(f, "%s%.*X", style, w, wordArray[i]);
      if((i + 1) != words) fprintf(f, ",");
   }
   return this;
}

//Translating
CharData*
CharData::translate()
{
   if(!lines) return NULL; //no image

   int i;                     //index to character image
   int n = lines * charWidth; //size of character image
   int mapIndex = 0;          //index to map 
   int wordIndex = 0;         //index to word array

   for(i = 0; i < n; i++)
   {
      if(image[i])
      {
         int k = wordIndex+mapTo[mapIndex];
         if(k >= MAX_WORDS_PER_CHAR)
         {
            printf("Error: Number of words per character exceeded limit of %d\n", MAX_WORDS_PER_CHAR);
            return NULL;
         }
         wordArray[k] |= mapAs[mapIndex];
         words = 0;
      }   
      mapIndex++;
      if(mapIndex == mapSizeInBits)
      {
         mapIndex = 0;
         wordIndex += mapSizeInWords;
         words = wordIndex;
      }
   }
   if(!words) words = wordIndex + mapSizeInWords;
   return this;
}

